package info.markhillman.SOMColorSorter;

/**
 * Class: SomNode
 * Description: This class is the single node for the
 * Som. The node should be created for a multidimensional
 * grid which will be the detector grid for the Som. Each
 * Node contains a weight for each input value as well as a
 * way to compare the weights and input values.
 * Created by Mark on 08/01/2016.
 */
public class SomNode {

    //Weights for the SomNode
    private float[] weights;

    //Create the SomNode and initialise the weights
    public SomNode(int inputSize) {

        //Create the weight array determined by the input size
        weights = new float[inputSize];

        //Initially random weights
        randomiseWeights();
    }

    //Randomise the weights in the SomNode
    public void randomiseWeights() {
        for (int i = 0; i < weights.length; i++) {
            weights[i] = (float)Math.random();
        }
    }

    //Return the distance away from the input data is from this nodes
    //weights
    public float calculateDistance(float[] inputs) {

        //Make sure that there are as many weights as there is input nodes
        if (inputs.length == weights.length) {

            //Calculate the sum of the inputs
            float distance = 0;
            for (int i = 0; i < weights.length; i++) {
                float difference = inputs[i] - weights[i];
                distance += difference * difference;
            }

            return distance;
        }
        else
            return 0f;
    }

    //Update this SomNode based off of its inputs and a learning rate
    public void update(float[] inputs, float learningRate) {

        //Iterate through the weights and update them
        for (int i = 0; i < weights.length; i++) {
            weights[i] += learningRate * (inputs[i] - weights[i]);
            if (weights[i] > 1)
                weights[i]= 1;
            if (weights[i] < 0)
                weights[i] = 0;
        }
    }

    //Get the weights
    public float[] getWeights() {
        return weights;
    }
}
